@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 """The readline based xonsh shell."""
 import os
+import sys
 import time
 import select
 import builtins
@@ -13,7 +14,7 @@ from xonsh.base_shell import BaseShell
 from xonsh.ansi_colors import partial_color_format, color_style_names, color_style
 from xonsh.environ import partial_format_prompt, multiline_prompt
 from xonsh.tools import print_exception
-from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS
+from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS, ON_CYGWIN
 
 if HAS_PYGMENTS:
     from xonsh import pyghooks
@@ -40,7 +41,8 @@ def setup_readline():
     import ctypes
     import ctypes.util
     readline.set_completer_delims(' \t\n')
-    if not readline.__file__.endswith('.py'):
+    # Cygwin seems to hang indefinitely when querying the readline lib
+    if (not ON_CYGWIN) and (not readline.__file__.endswith('.py')):
         RL_LIB = lib = ctypes.cdll.LoadLibrary(readline.__file__)
         try:
             RL_COMPLETION_SUPPRESS_APPEND = ctypes.c_int.in_dll(
@@ -69,8 +71,12 @@ def setup_readline():
         readline.parse_and_bind("bind ^I rl_complete")
     else:
         readline.parse_and_bind("tab: complete")
-    # load custom user settings
-    readline.read_init_file()
+    # try to load custom user settings
+    try:
+        readline.read_init_file()
+    except Exception:
+        # this seems to fail with libedit
+        print_exception('xonsh: could not load readline default init file.')
 
 
 def teardown_readline():
@@ -402,7 +408,7 @@ class ReadlineHistoryAdder(Thread):
         hist = builtins.__xonsh_history__
         while self.wait_for_gc and hist.gc.is_alive():
             time.sleep(0.011)  # gc sleeps for 0.01 secs, sleep a beat longer
-        files = hist.gc.unlocked_files()
+        files = hist.gc.files()
         i = 1
         for _, _, f in files:
             try:
