@@ -20,6 +20,7 @@ Implementations:
 import os
 import re
 import sys
+import ast
 import string
 import ctypes
 import builtins
@@ -32,9 +33,9 @@ from collections import OrderedDict, Sequence, Set
 
 # adding further imports from xonsh modules is discouraged to avoid cirular
 # dependencies
-from xonsh.platform import (has_prompt_toolkit, win_unicode_console,
-                            DEFAULT_ENCODING, ON_LINUX, ON_WINDOWS)
-
+from xonsh.platform import (has_prompt_toolkit, scandir, win_unicode_console,
+                            DEFAULT_ENCODING, ON_LINUX, ON_WINDOWS,
+                            PYTHON_VERSION_INFO)
 if has_prompt_toolkit():
     import prompt_toolkit
 else:
@@ -56,7 +57,10 @@ DefaultNotGiven = DefaultNotGivenType()
 
 BEG_TOK_SKIPS = frozenset(['WS', 'INDENT', 'NOT', 'LPAREN'])
 END_TOK_TYPES = frozenset(['SEMI', 'AND', 'OR', 'RPAREN'])
-LPARENS = frozenset(['LPAREN', 'AT_LPAREN', 'BANG_LPAREN', 'DOLLAR_LPAREN'])
+RE_END_TOKS = re.compile('(;|and|\&\&|or|\|\||\))')
+LPARENS = frozenset(['LPAREN', 'AT_LPAREN', 'BANG_LPAREN', 'DOLLAR_LPAREN',
+                     'ATDOLLAR_LPAREN'])
+
 
 def _is_not_lparen_and_rparen(lparens, rtok):
     """Tests if an RPAREN token is matched with something other than a plain old
@@ -66,6 +70,34 @@ def _is_not_lparen_and_rparen(lparens, rtok):
     return rtok.type == 'RPAREN' and any(x != 'LPAREN' for x in lparens)
 
 
+def find_next_break(line, mincol=0, lexer=None):
+    """Returns the column number of the next logical break in subproc mode.
+    This function may be useful in finding the maxcol argument of subproc_toks().
+    """
+    if mincol >= 1:
+        line = line[mincol:]
+    if lexer is None:
+        lexer = builtins.__xonsh_execer__.parser.lexer
+    if RE_END_TOKS.search(line) is None:
+        return None
+    maxcol = None
+    lparens = []
+    lexer.input(line)
+    for tok in lexer:
+        if tok.type in LPARENS:
+            lparens.append(tok.type)
+        elif tok.type in END_TOK_TYPES:
+            if _is_not_lparen_and_rparen(lparens, tok):
+                lparens.pop()
+            else:
+                maxcol = tok.lexpos + mincol + 1
+                break
+        elif tok.type == 'ERRORTOKEN' and ')' in tok.value:
+            maxcol = tok.lexpos + mincol + 1
+            break
+    return maxcol
+
+
 def subproc_toks(line, mincol=-1, maxcol=None, lexer=None, returnline=False):
     """Excapsulates tokens in a source code line in a uncaptured
     subprocess ![] starting at a minimum column. If there are no tokens
@@ -222,6 +254,7 @@ def indent(instr, nspaces=4, ntabs=0, flatten=False):
     else:
         return outstr
 
+
 def get_sep():
     """ Returns the appropriate filepath separator char depending on OS and
     xonsh options set
@@ -231,7 +264,6 @@ def get_sep():
             os.sep)
 
 
-
 def fallback(cond, backup):
     """Decorator for returning the object if cond is true and a backup if cond is false.
     """
@@ -285,6 +317,19 @@ class redirect_stderr(_RedirectStream):
     _stream = "stderr"
 
 
+def executables_in(path):
+    """Returns a generator of files in `path` that the user could execute. """
+    if PYTHON_VERSION_INFO < (3, 5, 0):
+        for i in os.listdir(path):
+            name  = os.path.join(path, i)
+            if (os.path.exists(name) and os.access(name, os.X_OK) and \
+                                    (not os.path.isdir(name))):
+                yield i
+    else:
+        yield from (x.name for x in scandir(path)
+                    if x.is_file() and os.access(x.path, os.X_OK))
+
+
 def command_not_found(cmd):
     """Uses the debian/ubuntu command-not-found utility to suggest packages for a
     command that cannot currently be found.
@@ -303,27 +348,26 @@ def command_not_found(cmd):
 
 def suggest_commands(cmd, env, aliases):
     """Suggests alternative commands given an environment and aliases."""
-    suggest_cmds = env.get('SUGGEST_COMMANDS')
-    if not suggest_cmds:
+    if not env.get('SUGGEST_COMMANDS'):
         return
     thresh = env.get('SUGGEST_THRESHOLD')
     max_sugg = env.get('SUGGEST_MAX_NUM')
     if max_sugg < 0:
         max_sugg = float('inf')
-
     cmd = cmd.lower()
     suggested = {}
-    for a in builtins.aliases:
-        if a not in suggested:
-            if levenshtein(a.lower(), cmd, thresh) < thresh:
-                suggested[a] = 'Alias'
 
-    for d in filter(os.path.isdir, env.get('PATH')):
-        for f in os.listdir(d):
-            if f not in suggested:
-                if levenshtein(f.lower(), cmd, thresh) < thresh:
-                    fname = os.path.join(d, f)
-                    suggested[f] = 'Command ({0})'.format(fname)
+    for alias in builtins.aliases:
+        if alias not in suggested:
+            if levenshtein(alias.lower(), cmd, thresh) < thresh:
+                suggested[alias] = 'Alias'
+
+    for path in filter(os.path.isdir, env.get('PATH')):
+        for _file in executables_in(path):
+            if _file not in suggested \
+                    and levenshtein(_file.lower(), cmd, thresh) < thresh:
+                suggested[_file] = 'Command ({0})'.format(os.path.join(path, _file))
+
     suggested = OrderedDict(
         sorted(suggested.items(),
                key=lambda x: suggestion_sort_helper(x[0].lower(), cmd)))
@@ -344,7 +388,7 @@ def suggest_commands(cmd, env, aliases):
     return rtn
 
 
-def print_exception():
+def print_exception(msg=None):
     """Print exceptions with/without traceback."""
     env = getattr(builtins, '__xonsh_env__', os.environ)
     if 'XONSH_SHOW_TRACEBACK' not in env:
@@ -356,6 +400,9 @@ def print_exception():
         exc_type, exc_value, exc_traceback = sys.exc_info()
         exception_only = traceback.format_exception_only(exc_type, exc_value)
         sys.stderr.write(''.join(exception_only))
+    if msg:
+        msg = msg if msg.endswith('\n') else msg + '\n'
+        sys.stderr.write(msg)
 
 
 # Modified from Public Domain code, by Magnus Lie Hetland
@@ -536,6 +583,26 @@ def to_bool_or_break(x):
         return to_bool(x)
 
 
+def is_bool_or_int(x):
+    """Returns whether a value is a boolean or integer."""
+    return is_bool(x) or is_int(x)
+
+
+def to_bool_or_int(x):
+    """Converts a value to a boolean or an integer."""
+    if isinstance(x, str):
+        return int(x) if x.isdigit() else to_bool(x)
+    elif is_int(x):  # bools are ints too!
+        return x
+    else:
+        return bool(x)
+
+
+def bool_or_int_to_str(x):
+    """Converts a boolean or integer to a string."""
+    return bool_to_str(x) if is_bool(x) else str(x)
+
+
 def ensure_int_or_slice(x):
     """Makes sure that x is list-indexable."""
     if x is None:
@@ -688,6 +755,36 @@ def is_history_tuple(x):
     return False
 
 
+def is_dynamic_cwd_width(x):
+    """ Determine if the input is a valid input for the DYNAMIC_CWD_WIDTH
+    environement variable.
+    """
+    return isinstance(x, tuple) and len(x) == 2 and isinstance(x[0], float) and \
+           (x[1] in set('c%'))
+
+
+def to_dynamic_cwd_tuple(x):
+    """Convert to a canonical cwd_width tuple."""
+    unit = 'c'
+    if isinstance(x, str):
+        if x[-1] == '%':
+            x = x[:-1]
+            unit = '%'
+        else:
+            unit = 'c'
+        return (float(x), unit)
+    else:
+        return (float(x[0]), x[1])
+
+
+def dynamic_cwd_tuple_to_str(x):
+    """Convert a canonical cwd_width tuple to a string."""
+    if x[1] == '%':
+        return str(x[0]) + '%'
+    else:
+        return str(x[0])
+
+
 RE_HISTORY_TUPLE = re.compile('([-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?)\s*([A-Za-z]*)')
 
 def to_history_tuple(x):
@@ -1074,8 +1171,7 @@ class CommandsCache(Set):
             return self._cmds_cache
         allcmds = set()
         for path in paths:
-            allcmds |= set(x for x in os.listdir(path)
-                           if os.path.isfile(x) and os.access(x, os.X_OK))
+            allcmds |= set(executables_in(path))
             allcmds |= set(builtins.aliases)
         self._cmds_cache = frozenset(allcmds)
         return self._cmds_cache
